import {Component, OnInit} from '@angular/core';
import {fadeIn} from '../../animation/fade-in';
import {ApiService} from '../../service/api';
import {NzMessageService, NzModalService} from 'ng-zorro-antd';
import {Router} from '@angular/router';
import {TranslateService} from '@ngx-translate/core';
import {CheckService} from '../../service/check';

@Component({
  selector: 'app-gateway',
  templateUrl: './gateway.component.html',
  styles: [`
    .btnPart {
      padding: 10px;
    }
  `],
  animations: [fadeIn]
})
export class GatewayComponent implements OnInit {

  data = [];
  query = {
    gatewayName: '',
    gatewaySN: ''
  };
  isLoading = false;
  isAllCheck = false;
  isIndeterminate = false;
  listOfBind = [{text: '已绑定', value: 1}, {text: '未绑定', value: 0}];
  selectedOne: any = {};
  isAddConV = false;
  isMultiAddConV = false;
  isUpdate = false;
  isBindConV = false;
  isDivideConV = false;
  isMulti = false;
  canbind = false;
  canUnbind = false;
  isCheckedOne = false;
  req: any = {};
  companys = [];
  influxdbs = [];
  filterFlag = null;
  selectedGateways = [];
  canBindGateways = [];
  req3: any = {};
  scenes = [{sceneNo: '123', sceneName: 'qq'}];
  req4: any = {};
  try = {};

  constructor(private api: ApiService, private message: NzMessageService, private router: Router,
              private modalService: NzModalService, private translate: TranslateService,
              private ck: CheckService) {
  }

  ngOnInit() {
    this.init();
  }

  init() {
    this.getData();
    this.getCompanys();
  }

  next = (data) => {
    if (data && data.constructor === Array) {
      const flag = !(this.filterFlag === 0 || this.filterFlag === 1);
      this.canBindGateways =  data.filter(one => !one.pDeviceNo);
      this.data = data.filter(one => (one.pDeviceNo && this.filterFlag || flag)
        && one.name.indexOf(this.query.gatewayName) > -1 && one.sn.indexOf(this.query.gatewaySN) > -1);
    } else if (data.code === 1) {
      this.message.success(data && data.message ? data.message : '操作成功');
      this.isAddConV = false;
      this.isMultiAddConV = false;
      this.isBindConV = false;
      this.isDivideConV = false;
      this.getData();
    } else if (data.code === 0) {
      this.message.error(data && data.message ? data.message : '操作失败');
    } else {
      this.message.error(data && data.message ? data.message : '获取数据失败');
    }
    this.isLoading = false;
  }

  getData() {
    this.isLoading = true;
    this.api.device.getAll([{model: 0}, this.next]);
  }

  getCompanys() {
    this.isLoading = true;
    this.api.company.getAll(['', (data) => {
      if (data && data.constructor === Array) {
        this.companys = data;
      } else {
        this.message.error(data && data.message ? data.message : '获取数据失败');
      }
      this.isLoading = false;
    }]);
  }

  companySelected(e) {
    this.isLoading = true;
    this.api.influxdbTables.getByCompanyNo([{companyNo: e}, (data) => {
      if (data && data.constructor === Array) {
        this.influxdbs = data;
      } else {
        this.message.error(data && data.message ? data.message : '获取数据失败');
      }
      this.isLoading = false;
    }]);
  }

  onSearch() {
    this.filterFlag = null;
    this.listOfBind = [{text: '已绑定', value: 1}, {text: '未绑定', value: 0}];
    this.getData();
  }

  onGetAll() {
    this.query = {
      gatewayName: '',
      gatewaySN: ''
    };
    this.filterFlag = null;
    this.listOfBind = [{text: '已绑定', value: 1}, {text: '未绑定', value: 0}];
    this.getData();
  }

  onInput(e) {
    this.selectedGateways = this.canBindGateways.filter(one => one.name.indexOf(e) > -1 || one.sn.indexOf(e) > -1);
  }

  // 复选框的方法
  initCheckArray(b) {
    if (b) {
      this.isCheckedOne = true;
    }
    this.data.map((one) => one.checked = b);
  }

  checkAll(e) {
    this.isIndeterminate = false;
    this.initCheckArray(e);
    this.checkCanBind();
  }

  oneCheck() {
    this.isCheckedOne = true;
    this.isIndeterminate = false;
    this.isAllCheck = false;
    const sum = this.data.filter((one) => one.checked).length;
    if (sum === this.data.length) {
      this.isAllCheck = true;
    } else if (sum === 0) {
      this.isCheckedOne = false;
      this.canbind = false;
      this.canUnbind = false;
    } else {
      this.isIndeterminate = true;
    }
    this.checkCanBind();
  }

  checkCanBind() {
    const obj = this.data.filter(one => one.checked);
    if (obj.length === 0) {
      this.canbind = false;
      this.canUnbind = false;
    } else {
      const flag = obj[0].pDeviceNo && true;
      if (obj.filter(one => (obj[0].pDeviceNo && true) === flag).length === obj.length) {
        this.canbind = flag;
        this.canUnbind = !flag;
      } else {
        this.canbind = false;
        this.canUnbind = false;
      }
    }
  }

  // 筛选
  filter(e) {
    this.filterFlag = e;
    this.getData();
  }

  // 跳转详情
  toSee(e) {
    this.router.navigate(['/app/deviceCenter/gatewayMgr/gatewaySee'],
      {queryParams: {deviceNo: e.deviceNo, name: e.name}});
  }

  // 单个的方法
  add() {
    this.req = {};
    this.isMulti = false;
    this.isUpdate = false;
    this.isAddConV = true;
    this.try = {field: ['name', 'sn', 'companyNo']};
  }

  bind(one) {
    this.isMulti = false;
    this.selectedGateways = [];
    this.req3 = {
      address: '',
      deviceNo: one.deviceNo,
      influxdbDatabaseNo: '',
      pDeviceNo: ''
    };
    this.try = {field: ['address', 'deviceNo', 'pDeviceNo']};
    this.isBindConV = true;
  }

  unbind(one) {
    this.modalService.confirm({
      nzTitle  : '<i> </i>',
      nzContent: '<b>' + this.translate.instant('confirmUnbindThisRecord') + '</b>',
      nzOnOk   : () => this.unbindOne(one)
    });
  }

  unbindOne(one) {
    this.isLoading = true;
    this.api.device.unBindDevice([{deviceNo: one.deviceNo}, this.next]);
  }

  update(one) {
    this.isMulti = false;
    this.req = {...one};
    this.companySelected(one.companyNo);
    this.try = {field: ['name', 'sn', 'companyNo', 'deviceNo']};
    this.isUpdate = true;
    this.isAddConV = true;
  }

  divide(one) {
    this.isMulti = false;
    this.req4 = {...one};
    this.try = {field: ['companyNo', 'deviceNo']};
    this.isDivideConV = true;
  }

  cancel(v) {
    this[v] = false;
  }

  save(v) {
    this.isLoading = true;
    if (!this.ck.formCheck(this[v], this.try)) {
      this.isLoading = false;
      return;
    }
    // 单个新增
    if (!this.isMulti && this.isAddConV && !this.isUpdate) {
      this.api.device.addGateway([this.req, this.next]);
      // 单个编辑
    } else if (!this.isMulti && this.isAddConV && this.isUpdate) {
      this.api.device.updateGateway([this.req, this.next]);
      // 单个分配
    } else if (!this.isMulti && this.isDivideConV) {
      this.api.device.updateCompany([this.req4, this.next]);
      // 单个绑定
    } else if (!this.isMulti && this.isBindConV) {
      this.req3.pDeviceNo = this.req3.device.deviceNo;
      this.req3.influxdbDatabaseNo = this.req3.device.influxdbDatabaseNo;
      this.api.device.gatewayBindGateway([this.req3, this.next]);
    }
  }

  deleteConfirm(one) {
    this.modalService.confirm({
      nzTitle  : '<i> </i>',
      nzContent: '<b>' + this.translate.instant('confirmDeleteThisRecord') + '</b>',
      nzOnOk   : () => this.delete(one)
    });
  }

  delete(one) {
    this.isLoading = true;
    this.api.device.delDevice([{deviceNo: one.deviceNo}, this.next]);
  }

  // 批量的方法
  allUpdate() {
    this.isMulti = true;
    this.isUpdate = true;
  }

  multiAdd() {
    this.isMulti = true;
    this.isMultiAddConV = true;
  }

  allDivide() {
    this.isMulti = true;
    this.isDivideConV = true;
  }

  allBind() {
    this.isMulti = true;
    this.isBindConV = true;
  }

  allDelete() {
    this.modalService.confirm({
      nzTitle  : '<i> </i>',
      nzContent: '<b>' + this.translate.instant('confirmDeleteSelectedRecord') + '</b>',
      nzOnOk   : () => this.deleteAll()
    });
  }

  deleteAll() {}

  allUnbind() {
    this.modalService.confirm({
      nzTitle  : '<i> </i>',
      nzContent: '<b>' + this.translate.instant('confirmUnbindSelectedRecord') + '</b>',
      nzOnOk   : () => this.unbindAll()
    });
  }

  unbindAll() {}

  toggleOn(one) {
    one.isLoading = true;
    this.api.device.mountUninstallDevice([{deviceNo: one.deviceNo, state: one.state === 0 ? 1 : 0}, (data) => {
      if (data && data.code === 1) {
        one.state = one.state === 0 ? 1 : 0;
        this.message.success(data && data.message ? data.message : '操作成功');
      } else {
        this.message.error(data && data.message ? data.message : '操作失败');
      }
      one.isLoading = false;
    }]);
  }
}
