import { Component, OnInit } from '@angular/core';
import {ApiService} from '../../service/api';
import {NzMessageService, NzModalService} from 'ng-zorro-antd';
import {Router} from '@angular/router';
import {TranslateService} from '@ngx-translate/core';
import {fadeIn} from '../../animation/fade-in';
import {CheckService} from '../../service/check';

@Component({
  selector: 'app-sensor-mgr',
  templateUrl: './sensor-mgr.component.html',
  styles: [`
    .btnPart {
      padding: 10px;
    }
  `],
  animations: [fadeIn]
})
export class SensorMgrComponent implements OnInit {

  data = [];
  query = {
    sensorName: '',
    sensorSN: ''
  };
  isLoading = false;
  isAllCheck = false;
  isIndeterminate = false;
  listOfBind = [{text: '已绑定', value: 1}, {text: '未绑定', value: 0}];
  isAddConV = false;
  isMultiAddConV = false;
  isBindConV = false;
  isMulti = false;
  isDivideConV = false;
  isUpdate = false;
  canbind = false;
  canUnbind = false;
  isCheckedOne = false;
  req: any = {};
  topicModels = [];
  companys = [];
  influxdbs = [];
  scenes = [{sceneNo: '123', sceneName: 'qq'}];
  filterFlag;
  selectedGateways = [];
  canBindGateways = [];
  req3: any = {};
  req5: any = {};
  try = {};

  constructor(private api: ApiService, private message: NzMessageService, private router: Router,
              private modalService: NzModalService, private translate: TranslateService, private ck: CheckService) {
  }

  ngOnInit() {
    this.init();
  }

  init() {
    this.getData();
    this.getTopicModels();
    this.getCompanys();
    this.getGateways();
  }

  next = (data) => {
    if (data && data.constructor === Array) {
      const flag = !(this.filterFlag === 0 || this.filterFlag === 1);
      this.data = data.filter(one => (one.pDeviceNo && this.filterFlag || flag)
       && one.name.indexOf(this.query.sensorName) > -1 && one.sn.indexOf(this.query.sensorSN) > -1);
    } else if (data.code === 1) {
      this.message.success(data && data.message ? data.message : '操作成功');
      this.isAddConV = false;
      this.isMultiAddConV = false;
      this.isBindConV = false;
      this.isDivideConV = false;
      this.getData();
    } else if (data.code === 0) {
      this.message.error(data && data.message ? data.message : '操作失败');
    } else {
      this.message.error(data && data.message ? data.message : '获取数据失败');
    }
    this.isLoading = false;
  }

  getData() {
    this.isLoading = true;
    this.api.device.getAll([{model: 1}, this.next]);
  }

  getTopicModels() {
    this.isLoading = true;
    this.api.topicModel.getAll(['', (data) => {
      if (data && data.constructor === Array) {
        this.topicModels = data;
      } else {
        this.message.error(data && data .message ? data.message : '获取数据失败');
      }
      this.isLoading = false;
    }]);
  }

  getCompanys() {
    this.isLoading = true;
    this.api.company.getAll(['', (data) => {
      if (data && data.constructor === Array) {
        this.companys = data;
      } else {
        this.message.error(data && data.message ? data.message : '获取数据失败');
      }
      this.isLoading = false;
    }]);
  }

  getGateways() {
    this.isLoading = true;
    this.api.device.getAll([{model: 0}, (data) => {
      if (data && data.constructor === Array) {
        this.canBindGateways =  data.filter(one => !one.pDeviceNo);
      } else {
        this.message.error(data && data.message ? data.message : '获取数据失败');
      }
      this.isLoading = false;
    }]);
  }

  onSearch() {
    this.filterFlag = null;
    this.listOfBind = [{text: '已绑定', value: 1}, {text: '未绑定', value: 0}];
    this.getData();
  }

  onGetAll() {
    this.query = {
      sensorName: '',
      sensorSN: ''
    };
    this.filterFlag = null;
    this.listOfBind = [{text: '已绑定', value: 1}, {text: '未绑定', value: 0}];
    this.getData();
  }

  onInput(e) {
    this.selectedGateways = this.canBindGateways.filter(one => one.name.indexOf(e) > -1 || one.sn.indexOf(e) > -1);
  }

  // 复选框的方法
  initCheckArray(b) {
    if (b) {
      this.isCheckedOne = true;
    }
    this.data.map((one) => one.checked = b);
  }

  checkAll(e) {
    this.isIndeterminate = false;
    this.initCheckArray(e);
    this.checkCanBind();
  }

  oneCheck() {
    this.isCheckedOne = true;
    this.isIndeterminate = false;
    this.isAllCheck = false;
    const sum = this.data.filter((one) => one.checked).length;
    if (sum === this.data.length) {
      this.isAllCheck = true;
    } else if (sum === 0) {
      this.isCheckedOne = false;
      this.canbind = false;
      this.canUnbind = false;
    } else {
      this.isIndeterminate = true;
    }
    this.checkCanBind();
  }

  checkCanBind() {
    const obj = this.data.filter(one => one.checked);
    if (obj.length === 0) {
      this.canbind = false;
      this.canUnbind = false;
    } else {
      const flag = obj[0].isOn;
      if (obj.filter(one => one.isOn === flag).length === obj.length) {
        this.canbind = flag;
        this.canUnbind = !flag;
      } else {
        this.canbind = false;
        this.canUnbind = false;
      }
    }
  }

  // 筛选
  filter(e) {
    this.filterFlag = e;
    this.getData();
  }

  companySelected(e) {
    this.isLoading = true;
    this.api.influxdbTables.getByCompanyNo([{companyNo: e}, (data) => {
      if (data && data.constructor === Array) {
        this.influxdbs = data;
      } else {
        this.message.error(data && data.message ? data.message : '获取数据失败');
      }
      this.isLoading = false;
    }]);
  }

  // 跳转详情
  toSee(e) {
    this.router.navigate(['/app/deviceCenter/sensorMgr/sensorDetail'], {queryParams: {deviceNo: e.deviceNo, name: e.name}});
  }

  // 单个的方法
  add() {
    this.req = {};
    this.isMulti = false;
    this.isAddConV = true;
    this.try = {field: ['name', 'sn', 'type', 'companyNo']};
  }

  bind(one) {
    this.isMulti = false;
    this.selectedGateways = [];
    this.req3 = {
      address: '',
      deviceNo: one.deviceNo,
      influxdbDatabaseNo: '',
      pDeviceNo: ''
    };
    this.isBindConV = true;
    this.try = {field: ['address', 'deviceNo', 'pDeviceNo']};
  }

  unbind(one) {
    this.modalService.confirm({
      nzTitle  : '<i></i>',
      nzContent: '<b>' + this.translate.instant('confirmUnbindThisRecord') + '</b>',
      nzOnOk   : () => this.unbindOne(one)
    });
  }

  unbindOne(one) {
    this.isLoading = true;
    this.api.device.unBindDevice([{deviceNo: one.deviceNo}, this.next]);
  }

  divide(one) {
    this.isMulti = false;
    this.req5 = {...one};
    this.try = {field: ['companyNo', 'deviceNo']};
    this.isDivideConV = true;
  }

  update(one) {
    this.isMulti = false;
    this.req = {...one};
    this.companySelected(one.companyNo);
    this.isUpdate = true;
    this.isAddConV = true;
    this.try = {field: ['name', 'sn', 'type', 'companyNo', 'deviceNo']};
  }

  cancel(v) {
    this[v] = false;
  }

  save(v) {
    this.isLoading = true;
    if (!this.ck.formCheck(this[v], this.try)) {
      this.isLoading = false;
      return;
    }
    if (!this.isMulti && this.isAddConV && !this.isUpdate) {
      this.req.alarmMethodNos = [];
      this.req.alarmUsers = [];
      this.req.model = 1;
      this.api.device.addSensorOrBlend([this.req, this.next]);
    } else if (!this.isMulti && this.isAddConV && this.isUpdate) {
      this.req.alarmMethodNos = [];
      this.req.alarmUsers = [];
      this.req.model = 1;
      this.api.device.updateSensor([this.req, this.next]);
    } else if (!this.isMulti && this.isBindConV) {
      this.req3.pDeviceNo = this.req3.device.deviceNo;
      this.req3.influxdbDatabaseNo = this.req3.device.influxdbDatabaseNo;
      this.api.device.gatewayBindGateway([this.req3, this.next]);
    } else if (!this.isMulti && this.isDivideConV) {
      this.api.device.updateCompany([this.req5, this.next]);
    }
  }

  deleteConfirm(one) {
    this.modalService.confirm({
      nzTitle  : '<i> </i>',
      nzContent: '<b>' + this.translate.instant('confirmDeleteThisRecord') + '</b>',
      nzOnOk   : () => this.delete(one)
    });
  }

  delete(one) {
    this.isLoading = true;
    this.api.device.delDevice([{deviceNo: one.deviceNo}, this.next]);
  }

  // 批量的方法
  allUpdate() {
    this.isMulti = true;
    this.isUpdate = true;
  }

  multiAdd() {
    this.isMulti = true;
    this.isMultiAddConV = true;
  }

  allDivide() {
    this.isMulti = true;
    this.isDivideConV = true;
  }

  allBind() {
    this.isMulti = true;
    this.isBindConV = true;
  }

  allDelete() {
    this.modalService.confirm({
      nzTitle  : '<i></i>',
      nzContent: '<b>' + this.translate.instant('confirmDeleteSelectedRecord') + '</b>',
      nzOnOk   : () => this.deleteAll()
    });
  }

  deleteAll() {}

  allUnbind() {
    this.modalService.confirm({
      nzTitle  : '<i></i>',
      nzContent: '<b>' + this.translate.instant('confirmUnbindSelectedRecord') + '</b>',
      nzOnOk   : () => this.unbindAll()
    });
  }

  unbindAll() {}

  toggleOn(one) {
    one.isLoading = true;
    this.api.device.mountUninstallDevice([{deviceNo: one.deviceNo, state: one.state === 0 ? 1 : 0}, (data) => {
      if (data && data.code === 1) {
        one.state = one.state === 0 ? 1 : 0;
        this.message.success(data && data.message ? data.message : '操作成功');
      } else {
        this.message.error(data && data.message ? data.message : '操作失败');
      }
      one.isLoading = false;
    }]);
  }
}
